var button = (function (exports) {
  'use strict';

  function getSubElements(subElementIds, svg) {
    var subElements = {};
    for (var i = 0; i < subElementIds.length; i++) {
      var id = subElementIds[i];
      var elem = svg.getElementById(id);

      if (!elem) {
        console.error("No element found with the " + id);
        continue;
      }

      subElements[id] = elem;
    }
    return subElements;
  }

  function init(svg, opts, subElements) {

    var onMouseDown = opts.onMouseDown;
    var onMouseUp = opts.onMouseUp;
    var onTouchStart = opts.onTouchStart ? opts.onTouchStart : onMouseDown;
    var onTouchEnd = opts.onTouchEnd ? opts.onTouchEnd : onMouseUp;
    var subElementIds = opts.subElementIds;

    function mousedown(e) {
      if (e.which !== 1 || "button" in e && e.button !== 0) return;

      console.log("mouse down");
      window.addEventListener("mouseup", mouseup);
      if (onMouseDown) {
        onMouseDown(e, subElements, svg);
      }
    }

    function mouseup(e) {
      console.log("mouse up");
      window.removeEventListener("mouseup", mouseup);
      if (onMouseUp) {
        onMouseUp(e, subElements, svg);
      }
    }

    function touchstart(e) {
      e.preventDefault();
      console.log("touch down");
      window.addEventListener("touchend", touchend);
      if (onTouchStart) {
        onTouchStart(e, subElements, svg);
      }
    }

    function touchend(e) {
      e.preventDefault();
      console.log("touch up");
      window.removeEventListener("touchend", touchend);
      if (onTouchEnd) {
        onTouchEnd(e, subElements, svg);
      }
    }

    svg.addEventListener("mousedown", mousedown);
    svg.addEventListener("touchstart", touchstart);
  }

  // API

  function connect(svgId) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var svgElem = void 0;
    if (typeof svgId === "string") {
      svgElem = document.getElementById(svgId);

      var subElements = void 0;
      if (opts.subElementIds) subElements = getSubElements(opts.subElementIds, svgElem);

      if (svgElem instanceof SVGSVGElement || svgElem instanceof SVGGElement) return init(svgElem, opts, subElements);else throw "Given element is not an <svg> or <g> element";
    } else {
      throw "The first argument must be a string (id of the SVG elem in the DOM";
    }
  }

  function attach(elem, targetElem) {
    var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var svgElem = void 0;
    if (elem instanceof SVGSVGElement) {
      svgElem = elem;
    } else if (elem instanceof SVGGElement) {
      svgElem = document.createElementNS("http://www.w3.org/2000/svg", "svg");
      svgElem.appendChild(elem);
    } else {
      throw "First argument must be an <svg> or a <g> element";
    }

    var subElements = void 0;
    if (opts.subElementIds) subElements = getSubElements(opts.subElementIds, svgElem);

    if (opts.preMount) {
      opts.preMount(subElements, svgElem);
    }

    targetElem.appendChild(elem);

    return init(svgElem, opts, subElements);
  }

  exports.connect = connect;
  exports.attach = attach;

  return exports;

}({}));
